import unittest

'''
Prgram 명: 미국 자산별 Business부문의 technical Analysis 만들기
Author: jeongmin Kang
Mail: jarvisNim@gmail.com
국가별 Economic 사이클을 분석하고, 자산시장별 금융환경 분석 그리고 ETF를 통한 섹터별 투자기회를 탐색하는 목적임.  
Technical Analysis 는 과거부터 현재까지의 이력을 기반으로 기술분석을 하는 개념임. 미래의 예측영역은 다른 프로그램에서 검토하도록 함.
- country: 잠재성장률대비 real GDP YoY, (include KR.Export) = nominal GDP - CPI
- market: Nasdaq, S&P500, KOSPI, KOSDAQ, US 3Y/10Y/20Y BOND, KR 1Y BOND, GOLD, OIL, COLLAR, YEN, WON, EURO 
- business: 각 섹터별 ETF
History
- 20231204  Create
- 20240220  change calulate trend into read Alpha table
'''

import sys, os
utils_dir = os.getcwd() + '/batch/utils'
sys.path.append(utils_dir)

from settings import *

import requests
import yfinance as yf
import pandas_ta as ta
import pyminerva as mi

from bs4 import BeautifulSoup as bs
from scipy import signal


'''
0. 공통영역 설정
'''
# logging
logger.warning(sys.argv[0] + ' :: ' + str(datetime.today()))
logger2.info(sys.argv[0] + ' :: ' + str(datetime.today()))

# 3개월단위로 순차적으로 읽어오는 경우의 시작/종료 일자 셋팅
to_date_2 = pd.to_datetime(to_date2)
three_month_days = relativedelta(weeks=12)
from_date = (to_date_2 - three_month_days).date()
to_date_2 = to_date_2.date()

database = 'Economics.db'
db_file = 'database/' + database
conn, engine = create_connection(db_file)

# TECH_LIST = ['sma','ema','macd','adx','psar','ichmoku',  # Trend
#            'rsi','stoch','roc','cci','willr','ao','stochrsi','ppo',  # Monentum
#            'obv','pvt','pvi','cmf','vwap','adosc','mfi','kvo','nvi',  # Volume
#            'atr','bbands','donchian','kc','rvi',]  # Volatility

# def get_indics(tech_type):
#     trend_indics = ['sma','ema','macd','adx','psar','ichmoku','obv','vwap','adosc','atr','donchian',
#                     'rvi',]
#     overBoughtSold_indics = ['rsi','stoch','willr','mfi','pvi',]
#     trendReversal_indics = ['roc','ao','nvi','bbands','kc',]
#     kReversal_indics = ['kvo','cmf',]

#     if tech_type in trend_indics:
#         result = 'trend'
#     elif tech_type in overBoughtSold_indics:
#         result = 'overBoughtSold'
#     elif tech_type in trendReversal_indics:
#         result = 'trendReversal'
#     elif tech_type in kReversal_indics:
#         result = 'kReversal'
#     else:
#         result = None

#     return result

# def add_month(to_date2:str, term_month:int):
#     target_date = pd.to_datetime(to_date2)
#     term_month = relativedelta(weeks=term_month*4)
#     target_date2 = (target_date + term_month).date()
#     return target_date2

# # Loading data, and split in train and test datasets
# def get_data(ticker, window):

#     ticker = yf.Ticker(ticker)
#     df = ticker.history(period='36mo') # test: 10mo, real: 36mo
#     try:
#         df['feature'] = signal.detrend(df['Close'])
#     except ValueError as e: # ValueError: cannot reshape array of size 0 into shape (0,newaxis)
#             sleep(5)                
#             logger2.error(f'get_data Value error 1: {e}')
#             df = ticker.history(period='36mo') # test: 10mo, real: 36mo
#     df['mean'] = df['feature'].rolling(window=window).mean()    
#     df['std'] = df['feature'].rolling(window=window).std()
    
#     return df


# def make_plot(ticker, df, tech_type, TECH_LIST, idx):
    
#     _df = df.copy() # 그래프만 그리면 되고, 본 df 에는 영향을 주지않기위해 복사본 만듬.
#     _df = _df.dropna()

#     h_limit = 70  # 밴드형 상단 기준선
#     l_limit = 30  # 밴드형 하단 기준선
#     base_limit = 0  # 기준선형 기준선

#     plt.subplot(len(TECH_LIST), 1, idx)    

#     max_val = max(_df[tech_type])
#     min_val = min(_df[tech_type])

#     if tech_type in ['sma','ema','macd','adx','ppo','psar','ichmoku','stoch','roc','ao',
#                      'obv','pvt','pvi','cmf','vwap','adosc','kvo','nvi','atr','rvi',]:  # 기준선형 그래프로
#         if tech_type == 'adx':
#             offset = 25  # offset: DI+ 와 DI- 차이가 25% 이상 또는 이하이면 signal ON
#             _df[tech_type] = _df[tech_type] - offset
#             plt.plot(_df.index, _df['DMP_14'], color='cyan')
#             plt.plot(_df.index, _df['DMN_14'], color='cyan')
#         elif tech_type == 'stoch':
#             h_limit = 85
#             l_limit = 15            
#             plt.plot(_df.index, _df['STOCHk_14_3_3'], color='cyan')
#             plt.plot(_df.index, _df['STOCHd_14_3_3'], color='cyan')
#             plt.axhline(y=h_limit, linestyle='--', lw=1.4, color='gray',)
#             plt.axhline(y=l_limit, linestyle='--', lw=1.4, color='gray',)            

#         if tech_type in ['obv','pvt','cmf','vwap','adosc']:  # Volume 베이스 들은 수치가 커서 지표그래프 안보임.
#             pass
#         else:
#             plt.plot(_df.index, _df['Close'])
#         plt.bar(_df.index, _df[tech_type], color=['g' if _df[tech_type].iloc[i] > base_limit else 'r' for i in range(len(_df))])

#         _df['pivot'] = np.where((_df[tech_type] > base_limit) & (_df[tech_type].shift(1) <= base_limit), 1, 0)  # Buy signal
#         _df['pivot'] = np.where((_df[tech_type] < base_limit) & (_df[tech_type].shift(1) >= base_limit), -1, _df['pivot']) # Sell signal
#         _1 = _df[_df['pivot'] == -1]
#         _2 = _df[_df['pivot'] == 1]

#         plt.axhline(y=base_limit, linestyle='--', color='red', linewidth=1)

#     else:  # top과 Buttom 구간값을 기준으로 설정하는 밴드형인경우    

#         if tech_type == 'cci':  
#             h_limit = 100
#             l_limit = -100
#         elif tech_type == 'willr':
#             h_limit = -20
#             l_limit = -80

#         plt.plot(_df.index, _df['Close'])
#         if tech_type == 'bbands':  # 
#             plt.plot(_df.index, _df['BBU_5_2.0'], color='orange')  # Upper line
#             plt.plot(_df.index, _df['BBL_5_2.0'], color='orange')  # Lower line
#             # print(_df)
#             _df['pivot'] = np.where((_df[tech_type] >= _df['BBU_5_2.0']) & (_df[tech_type].shift(1) <= _df['BBU_5_2.0']).shift(1), 1, 0)  # Sell signal
#             _df['pivot'] = np.where((_df[tech_type] <= _df['BBL_5_2.0']) & (_df[tech_type].shift(1) >= _df['BBL_5_2.0']).shift(1), -1, _df['pivot']) # Buy signal
#         elif tech_type == 'donchian':  # 
#             plt.plot(_df.index, _df['DCU_20_20'], color='orange')  # Upper line
#             plt.plot(_df.index, _df['DCL_20_20'], color='orange')  # Lower line
#             # print(_df)
#             _df['pivot'] = np.where((_df[tech_type] >= _df['DCU_20_20']) & (_df[tech_type].shift(1) <= _df['DCU_20_20']).shift(1), 1, 0)  # Sell signal
#             _df['pivot'] = np.where((_df[tech_type] <= _df['DCL_20_20']) & (_df[tech_type].shift(1) >= _df['DCL_20_20']).shift(1), -1, _df['pivot']) # Buy signal
#         elif tech_type == 'kc':  # 
#             plt.plot(_df.index, _df['KCUe_20_2'], color='orange')  # Upper line
#             plt.plot(_df.index, _df['KCLe_20_2'], color='orange')  # Lower line
#             _df['pivot'] = np.where((_df[tech_type] > _df['KCUe_20_2']) & (_df[tech_type].shift(1) <= _df['KCUe_20_2']).shift(1), 1, 0)  # Sell signal
#             _df['pivot'] = np.where((_df[tech_type] < _df['KCLe_20_2']) & (_df[tech_type].shift(1) >= _df['KCLe_20_2']).shift(1), -1, _df['pivot']) # Buy signal

#         else:
#             plt.plot(_df.index, _df[tech_type], color='orange')
#             plt.axhline(y=h_limit, linestyle='--', lw=1.4, color='gray',)
#             plt.axhline(y=l_limit, linestyle='--', lw=1.4, color='gray',)
#             _df['pivot'] = np.where((_df[tech_type] >= h_limit) & (_df[tech_type].shift(1) <= h_limit), 1, 0)  # Sell signal
#             _df['pivot'] = np.where((_df[tech_type] <= l_limit) & (_df[tech_type].shift(1) >= l_limit), -1, _df['pivot']) # Buy signal

#         _1 = _df[_df['pivot'] == 1]  # Sell signal
#         _2 = _df[_df['pivot'] == -1]  # Buy signal
#         for date in _1.index:  # Sell signal
#             plt.axvline(date, color='r', linestyle='--', lw=1,  alpha=1)  # Sell signal

#         for date in _2.index:  # Buy signal
#             plt.axvline(date, color='g', linestyle='--', lw=1,  alpha=1)  # Buy signal
 
#     signal = get_indics(tech_type)
#     # print(signal)

#     if _1.empty and _2.empty:
#         latest_date = '9999-99-99'
#         decision = '현상태 유지'
#     elif _1.empty:
#         latest_date = max(_2.index)
#         if signal == 'trend':
#             decision = '추세상승 시작'
#         elif signal == 'overBoughtSold':
#             decision = '과매도 시작'
#         elif signal == 'trendReversal':
#             decision = '바닥지나 상승 시작'
#         elif signal == 'trendReversal':
#             decision = 'k 상승반전 시작'            
#         else:
#             decision = '분할매수 시작'
#     elif _2.empty:
#         latest_date = max(_1.index)
#         if signal == 'trend':
#             decision = '추세하락 시작'
#         elif signal == 'overBoughtSold':
#             decision = '과매수 시작'
#         elif signal == 'trendReversal':
#             decision = '천정지나 하락 시작'
#         elif signal == 'trendReversal':
#             decision = 'k 하락반전 시작'            
#         else:
#             decision = '분할매도 시작'
#     else:
#         latest_date = max(_1.index[-1], _2.index[-1])
#         if latest_date == _1.index[-1]:
#             if signal == 'trend':
#                 decision = '추세하락 시작'
#             elif signal == 'overBoughtSold':
#                 decision = '과매수 시작'
#             elif signal == 'trendReversal':
#                 decision = '천정지나 하락 시작'
#             elif signal == 'trendReversal':
#                 decision = 'k 하락반전 시작'            
#             else:
#                 decision = '분할매도 시작'
#         else:
#             if signal == 'trend':
#                 decision = '추세상승 시작'
#             elif signal == 'overBoughtSold':
#                 decision = '과매도 시작'
#             elif signal == 'trendReversal':
#                 decision = '바닥지나 상승 시작'
#             elif signal == 'trendReversal':
#                 decision = 'k 상승반전 시작'            
#             else:
#                 decision = '분할매수 시작'

#     try:
#         latest_date = latest_date.date()
#     except:
#         pass
#     plt.title(f"{ticker}: {tech_type} / {latest_date}부터 {decision} ", fontdict={'fontsize':20, 'color':'g'})    
#     plt.grid(lw=0.7, color='lightgray')
#     # plt.xlabel(f'Last Date: {latest_date}', loc='right', fontsize=18)
    
#     return latest_date




# '''
# I. Technical Analysis

# - 1. Trend
#     Simple Moving Average (SMA)*
#     Exponential Moving Average (EMA)*
#     Weighted Moving Average (WMA)
#     Moving Average Convergence Divergence (MACD)
#     Average Directional Movement Index (ADX)*
#     Vortex Indicator (VI)
#     Trix (TRIX)
#     Mass Index (MI)
#     Detrended Price Oscillator (DPO)
#     KST Oscillator (KST)
#     Ichimoku Kinkō Hyō (Ichimoku)*
#     Parabolic Stop And Reverse (Parabolic SAR)*
#     Schaff Trend Cycle (STC)
#     ZigZag Indicator.... (Not yet)

    
# - 2. Momentum: https://medium.com/@crisvelasquez/top-6-momentum-indicators-in-python-bea0875e60a5
#     Relative Strength Index (RSI)*
#     Stochastic RSI (SRSI)
#     True strength index (TSI)
#     Ultimate Oscillator (UO)
#     Stochastic Oscillator (SR)*
#     Williams %R (WR)*
#     Awesome Oscillator (AO)*
#     Kaufman’s Adaptive Moving Average (KAMA)
#     Rate of Change (ROC)*
#     Percentage Price Oscillator (PPO)
#     Percentage Volume Oscillator (PVO)
#     Commodity Channel Index (CCI)*

    
# - 3. Volume
#     Money Flow Index (MFI)*
#     Accumulation/Distribution Index (ADI)*
#     On-Balance Volume (OBV)*
#     Volume-price Trend (VPT)*    
#     Chaikin Money Flow (CMF)*
#     Force Index (FI)
#     Ease of Movement (EoM, EMV)
#     Negative Volume Index (NVI)*
#     Volume Weighted Average Price (VWAP)*
#     Ulcer Index (UI)
#     Positive Volume Index (PVI)*
#     Klinger Volume Oscillator(KVO)*

    
# - 4. Volatility
#     Average True Range (ATR)*
#     Bollinger Bands (BB)*
#     Donchian Channel (DC)*
#     Keltner Channel (KC)*
#     Relative Volatility Index (RVI).... (Not yet)
#     Volatility Chaikin.... (Not yet)

# '''

# '''
# 1. Trend Indicators: https://medium.com/@crisvelasquez/top-6-trend-indicators-in-python-c922ac0674f9
# '''
# # Trend.Simple Moving Average (SMA)
# # calculates the average price over a specific period, providing a smoothed representation of price movement.
# # 특정 기간 동안의 평균 가격을 계산하여 가격 변동을 부드럽게 표현합니다.
# def sma(ticker, df, tech_type, TECH_LIST, idx):
#     df[tech_type] = df.ta.sma(20) - df.ta.sma(200)
#     latest_date = make_plot(ticker, df, tech_type, TECH_LIST, idx)
    
#     return df, latest_date

# # Trend.Exponential Moving Average (EMA)
# # gives more weight to recent prices, making it more responsive to new information.
# # 최근 가격에 더 많은 가중치를 부여하여 새로운 정보에 더 잘 반응합니다.
# def ema(ticker, df, tech_type, TECH_LIST, idx):
#     df[tech_type] = df.ta.ema(20) - df.ta.ema(200)
#     latest_date = make_plot(ticker, df, tech_type, TECH_LIST, idx)
    
#     return df, latest_date

# # Trend.Exponential Moving Average (EMA)
# # shows the relationship between two moving averages of a security’s price.
# # 증권 가격의 두 이동 평균 사이의 관계를 보여줍니다.
# def macd(ticker, df, tech_type, TECH_LIST, idx):
#     # Get the 26-day EMA of the closing price
#     k = df['Close'].ewm(span=12, adjust=False, min_periods=12).mean()
#     # Get the 12-day EMA of the closing price
#     d = df['Close'].ewm(span=26, adjust=False, min_periods=26).mean()
#     # Subtract the 26-day EMA from the 12-Day EMA to get the MACD
#     macd = k - d
#     # Get the 9-Day EMA of the MACD for the Trigger line
#     macd_s = macd.ewm(span=9, adjust=False, min_periods=9).mean()
#     # Calculate the difference between the MACD - Trigger for the Convergence/Divergence value
#     df[tech_type] = (macd - macd_s)*5  # 폭이 너무 작아 크게 보이기 위해 *5
#     latest_date = make_plot(ticker, df, tech_type, TECH_LIST, idx)
    
#     return df, latest_date

# # Average Directional Movement Index (ADX):
# # quantify the strength of a trend. 추세의 강도를 정량화합니다.
# # The +DI and -DI lines can help identify the direction of the trend. 양의 방향 표시기(+DI) 및 음의 방향 표시기(-DI).
# def adx(ticker, df, tech_type, TECH_LIST, idx):  # DMP_14(posituve) 와 DMN_14(negative) 두 개 칼럼 생성
#     df.ta.adx(length=14, append=True)
#     df[tech_type] = df['ADX_14']
#     latest_date = make_plot(ticker, df, tech_type, TECH_LIST, idx)
    
#     return df, latest_date

# # Parabolic Stop And Reverse (Parabolic SAR)
# def psar(ticker, df, tech_type, TECH_LIST, idx):  # DMP_14(posituve) 와 DMN_14(negative) 두 개 칼럼 생성
#     df.ta.psar(append=True)
#     df['PSARl_0.02_0.2'] = df['PSARl_0.02_0.2'].fillna(0)
#     df['PSARs_0.02_0.2'] = df['PSARs_0.02_0.2'].fillna(0)
#     df[tech_type] = df['Close'] - (df['PSARl_0.02_0.2'] + df['PSARs_0.02_0.2'])  # PSARl_0.02_0.2 는 long 값, PSARl_0.02_0.2 는 short 값
#     latest_date = make_plot(ticker, df, tech_type, TECH_LIST, idx)
    
#     return df, latest_date


# # Ichimoku Kinkō Hyō (Ichimoku)
# # a trend-following indicator that provides entry and exit points.
# # 진입점과 이탈점을 제공하는 추세추종 지표입니다.
# def ichmoku(ticker, df, tech_type, TECH_LIST, idx):
#     df.ta.ichimoku(append=True)
#     # Tenkan Sen: ITS_9 > Kijun Sen: IKS_26 => Buy signal
#     df[tech_type] = (df['ITS_9'] - df['IKS_26'])*5  # 폭이 너무 작아 크게 보이기 위해 *5
#     latest_date = make_plot(ticker, df, tech_type, TECH_LIST, idx)
    
#     return df, latest_date


# '''
# 2. Momentum Indicators: https://medium.com/@crisvelasquez/top-6-momentum-indicators-in-python-bea0875e60a5
# '''
# # Momentum.Relative Strength Index (RSI) 
# # measures the speed and change of price movements, oscillating between 0 and 100.
# # 0과 100 사이에서 진동하는 가격 움직임의 속도와 변화를 측정합니다.
# def rsi(ticker, df, tech_type, TECH_LIST, idx):    
#     df[tech_type] = df.ta.rsi(14)
#     latest_date = make_plot(ticker, df, tech_type, TECH_LIST, idx)
    
#     return df, latest_date

# # Momentum.Stochastic Oscillator (STOCH): 
# # comparing a particular closing price of a security to a range of its prices over a certain period of time.
# # 특정 기간 동안의 특정 유가 증권 종가를 해당 가격 범위와 비교하는 것입니다.
# def stoch(ticker, df, tech_type, TECH_LIST, idx):    
#     df.ta.stoch(append=True)
#     df[tech_type] = df['STOCHk_14_3_3'] - df['STOCHd_14_3_3']
#     # print(df.tail())
#     latest_date = make_plot(ticker, df, tech_type, TECH_LIST, idx)
    
#     return df, latest_date

# # Momentum.Rate of Change (ROC) Momentum Oscillator
# # measures the percentage change in price between the current price and the price a certain number of periods ago.
# # 현재 가격과 특정 기간 전 가격 사이의 가격 변화율을 측정합니다.
# def roc(ticker, df, tech_type, TECH_LIST, idx):    
#     df.ta.roc(append=True)
#     df[tech_type] = df['ROC_10']
#     # print(df.tail())
#     latest_date = make_plot(ticker, df, tech_type, TECH_LIST, idx)
    
#     return df, latest_date

# # Momentum.Commodity Channel Index (CCI): identifies cyclical trends in securities 
# # by comparing their current typical price (TP) to the average TP over a specific period, usually 20 days.
# # 현재 일반 가격(TP)을 특정 기간(보통 20일) 동안의 평균 TP와 비교하여 증권의 순환 추세를 식별합니다.
# def cci(ticker, df, tech_type, TECH_LIST, idx):    
#     df.ta.cci(append=True)
#     df[tech_type] = df['CCI_14_0.015']
#     # print(df.tail())
#     latest_date = make_plot(ticker, df, tech_type, TECH_LIST, idx)
    
#     return df, latest_date

# # Momentum.Williams %R (WR): reflects the level of the close relative to the highest high for a set period.
# # 일정 기간 동안 최고가 대비 종가 수준을 반영합니다.
# def willr(ticker, df, tech_type, TECH_LIST, idx):    
#     df.ta.willr(append=True)
#     df[tech_type] = df['WILLR_14']
#     # print(df.tail())
#     latest_date = make_plot(ticker, df, tech_type, TECH_LIST, idx)
    
#     return df, latest_date

# # Momentum.Awesome Oscillator (AO): designed to capture momentum in the market by comparing the recent market momentum with the general momentum over a wider frame of time.
# # 최근 시장 모멘텀과 더 넓은 기간 동안의 일반적인 모멘텀을 비교하여 시장의 모멘텀을 포착합니다.
# def ao(ticker, df, tech_type, TECH_LIST, idx):    
#     df.ta.ao(append=True)
#     df[tech_type] = df['AO_5_34']
#     # print(df.tail())
#     latest_date = make_plot(ticker, df, tech_type, TECH_LIST, idx)
    
#     return df, latest_date

# # Momentum.Stochastic RSI (STOCHRSI)
# def stochrsi(ticker, df, tech_type, TECH_LIST, idx):
#     df.ta.stochrsi(append=True)    
#     df[tech_type] = df['STOCHk_14_3_3'] - df['STOCHd_14_3_3']
#     # print(df.tail())
#     latest_date = make_plot(ticker, df, tech_type, TECH_LIST, idx)
    
#     return df, latest_date

# # Momentum.Percentage Price Oscillator (PPO):
# # "Stochastic RSI and Dynamic Momentum Index" was created by Tushar Chande and Stanley Kroll and published in Stock & Commodities V.11:5 (189-199)
# # It is a range-bound oscillator with two lines moving between 0 and 100.
# def ppo(ticker, df, tech_type, TECH_LIST, idx):    
#     buf= df.ta.ppo(close=df['Close'], append=True)
#     df[tech_type] = buf['PPO_12_26_9']
#     latest_date = make_plot(ticker, df, tech_type, TECH_LIST, idx)
                    
#     return df, latest_date

# '''
# 3. Volume Indicators: https://medium.com/@crisvelasquez/top-9-volume-indicators-in-python-e398791b98f9
# '''
# # Volume.On-Balance Volume (OBV)
# # a cumulative indicator that relates volume to price change. 
# # OBV increases or decreases during each trading day in line with whether the price closes higher or lower from the previous close.
# # 거래량과 가격 변화를 연결하는 누적 지표입니다.
# # OBV는 가격이 이전 종가보다 높게 또는 낮게 마감되는지 여부에 따라 각 거래일 동안 증가하거나 감소합니다.
# def obv(ticker, df, tech_type, TECH_LIST, idx):    
#     df.ta.obv(append=True)
#     df['OBV_EMA'] = df['OBV'].ewm(span=30).mean()  # 20-day EMA of OBV
#     df[tech_type] = df['OBV'] - df['OBV_EMA']
#     # print(df.tail())
#     latest_date = make_plot(ticker, df, tech_type, TECH_LIST, idx)
    
#     return df, latest_date

# # Volume.Volume Price Trend (VPT)
# # integrates volume with the percentage change in price, cumulatively indicating the trend’s strength and direction.
# # 거래량과 가격 변동률을 통합하여 추세의 강도와 방향을 누적적으로 나타냅니다.
# def pvt(ticker, df, tech_type, TECH_LIST, idx):    
#     df.ta.pvt(append=True)
#     df['PVT_MA'] = df['PVT'].rolling(window=30).mean()  # 20-day moving average
#     df[tech_type] = df['PVT'] - df['PVT_MA']
#     # print(df.tail())
#     latest_date = make_plot(ticker, df, tech_type, TECH_LIST, idx)
    
#     return df, latest_date

# # Volume.Positive Volume Index (PVI)
# # @@@quantifies the percentage rate at which volume changes over a specific period, comparing current volume to volume from n days ago.
# # @@@n일 전의 거래량과 현재 거래량을 비교하여 특정 기간 동안 거래량이 변경되는 비율을 정량화합니다.
# def pvi(ticker, df, tech_type, TECH_LIST, idx):    
#     df.ta.pvi(append=True)
#     df['PVI_1_MA'] = df['PVI_1'].rolling(window=30).mean()  # 20-day moving average
#     df[tech_type] = df['PVI_1'] - df['PVI_1_MA']
#     # print(df.tail())
#     latest_date = make_plot(ticker, df, tech_type, TECH_LIST, idx)
    
#     return df, latest_date

# # Volume.Chaikin Money Flow (CMF)
# # integrates price and volume to measure the buying and selling pressure over a specified period, typically 20 days.
# # 특정 기간(일반적으로 20일) 동안의 매수 및 매도 압력을 측정하기 위해 가격과 거래량을 통합합니다.
# def cmf(ticker, df, tech_type, TECH_LIST, idx):    
#     df.ta.cmf(append=True)
#     df[tech_type] = df['CMF_20']
#     # print(df.tail())
#     latest_date = make_plot(ticker, df, tech_type, TECH_LIST, idx)
    
#     return df, latest_date

# # Volume.Volume-Weighted Average Price (VWAP)
# # gives the average price a stock has traded at during the day, weighted by volume
# # 거래량에 따라 가중치를 적용하여 하루 동안 주식이 거래된 평균 가격을 제공합니다.
# def vwap(ticker, df, tech_type, TECH_LIST, idx):
#     df.ta.vwap(append=True)
#     df[tech_type] = df['Close'] - df['VWAP_D']
#     # print(df.tail())
#     latest_date = make_plot(ticker, df, tech_type, TECH_LIST, idx)
    
#     return df, latest_date

# # Volume.Accumulation/Distribution Oscillator (ADOSC)
# # designed to reflect the cumulative flow of money into or out of a security, factoring in both the volume and the price movement.
# # 거래량과 가격 변동을 모두 고려하여 증권 안팎으로의 누적 자금 흐름을 반영하도록 설계되었습니다.
# def adosc(ticker, df, tech_type, TECH_LIST, idx):
#     df.ta.adosc(append=True)
#     df[tech_type] = df['ADOSC_3_10']
#     # print(df.tail())
#     latest_date = make_plot(ticker, df, tech_type, TECH_LIST, idx)
    
#     return df, latest_date

# # Volume.Money Flow Index (MFI)
# # a technical momentum indicator that combines price and volume to assess the buying or selling pressure on an asset. 
# # 자산에 대한 매수 또는 매도 압력을 평가하기 위해 가격과 거래량을 결합하는 기술적 모멘텀 지표입니다.
# def mfi(ticker, df, tech_type, TECH_LIST, idx):
#     df[tech_type] = df.ta.mfi(close=df['Close'],length=14, append=True)
#     latest_date = make_plot(ticker, df, tech_type, TECH_LIST, idx)

#     return df, latest_date

# # Volume.Klinger Volume Oscillator (KVO)
# # It is designed to predict price reversals in a market by comparing volume to price
# # 거래량과 가격을 비교하여 시장의 가격 반전을 예측하도록 설계되었습니다.
# def kvo(ticker, df, tech_type, TECH_LIST, idx):
#     df.ta.kvo(append=True)
#     df[tech_type] = df['KVO_34_55_13']
#     # print(df.tail())
#     latest_date = make_plot(ticker, df, tech_type, TECH_LIST, idx)
    
#     return df, latest_date

# # Volume.Negative Volume Index (NVI)
# # cumulative indicator that increases when the volume decreases compared to the previous session, suggesting that the “smart money” is active.
# # 이전 세션에 비해 거래량이 감소할 때 증가하는 누적 지표로 '스마트머니'가 활성화되었음을 시사한다.
# def nvi(ticker, df, tech_type, TECH_LIST, idx):
#     df.ta.nvi(append=True)
#     df['NVI_1_SMA'] = df['NVI_1'].rolling(window=20).mean()    
#     df[tech_type] = df['NVI_1'] - df['NVI_1_SMA']
#     # print(df.tail())
#     latest_date = make_plot(ticker, df, tech_type, TECH_LIST, idx)
    
#     return df, latest_date


# '''
# 4. Volatility Indicators: https://medium.com/@crisvelasquez/top-6-volatility-indicators-in-python-14fbd7bf92d8
# '''
# # Volatility.Average True Range (ATR)
# # quantifies market volatility by averaging the range of price movements.
# # 가격 변동 범위를 평균하여 시장 변동성을 정량화합니다.
# def atr(ticker, df, tech_type, TECH_LIST, idx):
#     df.ta.atr(append=True)
#     df['ATRr_14_MA'] = df['ATRr_14'].rolling(window=20).mean()    
#     df[tech_type] = df['ATRr_14'] - df['ATRr_14_MA']
#     # print(df.tail())
#     latest_date = make_plot(ticker, df, tech_type, TECH_LIST, idx)
    
#     return df, latest_date

# # Volatility.Bollinger Bands (BB)
# # provide insights into market volatility and overbought/oversold conditions.
# # 시장 변동성과 과매수/과매도 상태에 대한 통찰력을 제공합니다.
# def bbands(ticker, df, tech_type, TECH_LIST, idx):    
#     df.ta.bbands(append=True)
#     df[tech_type] = df['Close']
#     # print(df.tail())
#     latest_date = make_plot(ticker, df, tech_type, TECH_LIST, idx)    
    
#     return df, latest_date

# # Volatility.Donchian Channels (DONCHAIN)
# # are based on the highest high and lowest low, offering a view of market range and volatility.
# # 최고가와 최저가를 기반으로 하며 시장 범위와 변동성에 대한 시각을 제공합니다.
# def donchian(ticker, df, tech_type, TECH_LIST, idx):    
#     df.ta.donchian(append=True)
#     df[tech_type] = df['Close']
#     # print(df.tail())
#     latest_date = make_plot(ticker, df, tech_type, TECH_LIST, idx)    
    
#     return df, latest_date

# # Volatility.Keltner Channels (KC)
# # use ATR for band setting, making them sensitive to volatility spikes.
# # 밴드 설정에 ATR을 사용하여 변동성 급증에 민감하게 만듭니다.
# def kc(ticker, df, tech_type, TECH_LIST, idx):    
#     df.ta.kc(append=True)
#     df[tech_type] = df['Close']
#     # print(df.tail())
#     latest_date = make_plot(ticker, df, tech_type, TECH_LIST, idx)    
    
#     return df, latest_date

# # Volatility.Relative Volatility Index (RVI)
# # volatility measurement that mirrors the RSI but uses standard deviation.
# # RSI를 반영하지만 표준편차를 사용하는 변동성 측정입니다.
# def rvi(ticker, df, tech_type, TECH_LIST, idx):
#     df.ta.rvi(append=True)
#     df['RVI_14_SMA'] = df['RVI_14'].rolling(window=20).mean()    
#     df[tech_type] = df['RVI_14'] - df['RVI_14_SMA']
#     # print(df.tail())
#     latest_date = make_plot(ticker, df, tech_type, TECH_LIST, idx)
    
#     return df, latest_date





'''
Main Fuction
'''

if __name__ == "__main__":


    '''
    0. 공통
    '''
    def tech_func(func, ticker, df, tech_type, TECH_LIST, idx):

        # df, latest_date = func(ticker, df, tech_type, TECH_LIST, idx)  # 모듈 호출방식으로 바뀌며 삭제하나, 유사 사용가능
        if func == 'sma':   df, latest_date = mi.sma(ticker, df, tech_type, TECH_LIST, idx)
        elif func == 'ema':   df, latest_date = mi.ema(ticker, df, tech_type, TECH_LIST, idx)
        elif func == 'macd':   df, latest_date = mi.macd(ticker, df, tech_type, TECH_LIST, idx)
        elif func == 'adx':   df, latest_date = mi.adx(ticker, df, tech_type, TECH_LIST, idx)
        elif func == 'psar':   df, latest_date = mi.psar(ticker, df, tech_type, TECH_LIST, idx)
        elif func == 'ichmoku':   df, latest_date = mi.ichmoku(ticker, df, tech_type, TECH_LIST, idx)
        elif func == 'rsi':   df, latest_date = mi.rsi(ticker, df, tech_type, TECH_LIST, idx)
        elif func == 'stoch':   df, latest_date = mi.stoch(ticker, df, tech_type, TECH_LIST, idx)
        elif func == 'roc':   df, latest_date = mi.roc(ticker, df, tech_type, TECH_LIST, idx)
        elif func == 'cci':   df, latest_date = mi.cci(ticker, df, tech_type, TECH_LIST, idx)
        elif func == 'willr':   df, latest_date = mi.willr(ticker, df, tech_type, TECH_LIST, idx)
        elif func == 'ao':   df, latest_date = mi.ao(ticker, df, tech_type, TECH_LIST, idx)
        elif func == 'stochrsi':   df, latest_date = mi.stochrsi(ticker, df, tech_type, TECH_LIST, idx)
        elif func == 'ppo':   df, latest_date = mi.ppo(ticker, df, tech_type, TECH_LIST, idx)
        elif func == 'obv':   df, latest_date = mi.obv(ticker, df, tech_type, TECH_LIST, idx)        
        elif func == 'pvt':   df, latest_date = mi.pvt(ticker, df, tech_type, TECH_LIST, idx)
        elif func == 'pvi':   df, latest_date = mi.pvi(ticker, df, tech_type, TECH_LIST, idx)
        elif func == 'cmf':   df, latest_date = mi.cmf(ticker, df, tech_type, TECH_LIST, idx)
        elif func == 'vwap':   df, latest_date = mi.vwap(ticker, df, tech_type, TECH_LIST, idx)
        elif func == 'adosc':   df, latest_date = mi.adosc(ticker, df, tech_type, TECH_LIST, idx)
        elif func == 'mfi':   df, latest_date = mi.mfi(ticker, df, tech_type, TECH_LIST, idx)
        elif func == 'kvo':   df, latest_date = mi.kvo(ticker, df, tech_type, TECH_LIST, idx)
        elif func == 'nvi':   df, latest_date = mi.nvi(ticker, df, tech_type, TECH_LIST, idx)
        elif func == 'atr':   df, latest_date = mi.atr(ticker, df, tech_type, TECH_LIST, idx)
        elif func == 'bbands':   df, latest_date = mi.bbands(ticker, df, tech_type, TECH_LIST, idx)
        elif func == 'donchian':   df, latest_date = mi.donchian(ticker, df, tech_type, TECH_LIST, idx)
        elif func == 'kc':   df, latest_date = mi.kc(ticker, df, tech_type, TECH_LIST, idx)
        elif func == 'rvi':   df, latest_date = mi.rvi(ticker, df, tech_type, TECH_LIST, idx)                                             
        else:   logger2.error(f"tech name is not valid.")
        
        return df, latest_date


    '''
    1. Technical Analysis
    '''

    M_table = 'Alpha'
    country = 'US'

    M_query = f"SELECT * from {M_table} WHERE Country = '{country}' GROUP BY Country, Market, Busi, Researcher"

    try:
        df = pd.read_sql_query(M_query, conn)
        df = df.drop(columns=['Date', 'Country_Growth', 'Market_Growth', 'Busi_Growth'])
        df = df.sort_values(['Country', 'Market', 'Busi', 'Researcher'], ascending=True).reset_index(drop=True)
        # print(df)
        df.columns = ['Country', 'Market', 'Busi', 'Researcher', 'Trend', 'Trend_3mo', 'Trend_6mo', 'Trend_12mo', 'Trend_18mo', 'Trend_24mo']

        melted_df = pd.melt(df, id_vars=['Country', 'Market', 'Busi', 'Researcher'],
                    var_name='Trend', value_name='Growth') # @@@ var_name=f'{col_0}'... 우연...
        # print(melted_df)
    except Exception as e:
        logger.error(' >>> plot_alpha_tickers Exception: {}'.format(e))

    tickers = df['Busi'].unique()

    # 현재일, 3개월, 6개월, 12개월, 18개월 뒤 날짜 셋팅
    # for month_term in month_terms:
    #     if month_term == 0:
    #         col_0 = add_month(to_date2, month_term)
    #     elif month_term == 3:
    #         col_3 = add_month(to_date2, month_term)
    #     elif month_term == 6:
    #         col_6 = add_month(to_date2, month_term)
    #     elif month_term == 12:
    #         col_12 = add_month(to_date2, month_term)
    #     elif month_term == 18:
    #         col_18 = add_month(to_date2, month_term)                                    
    #     else:
    #         col_24 = add_month(to_date2, month_term)    
    

    # ticker 별 각 테크니컬 분석
    for ticker in tickers:

        print(ticker)

        plt.figure(figsize=(16, 4*len(TECH_LIST)))
        try:
            df = mi.get_data(ticker, 20)
        except ValueError as e: # ValueError: cannot reshape array of size 0 into shape (0,newaxis)
            for i in range(2):
                sleep(5)                
                logger2.error(f'get_data Value error 2: {e}')
                df = mi.get_data(ticker, 20)


        for i, tech_name in enumerate(TECH_LIST):
            print('      ', tech_name)
            # # 문자열을 함수 이름으로 변환하여 함수 가져오기 -> 디렉토리 내에서는 가능하나, 패키지 모듈에서 호출하기는 어려움. ㅜㅜ
            # func = globals().get(func_name)
            func = tech_name
            df, latest_date = tech_func(func, ticker, df, tech_name, TECH_LIST, i+1)

        sleep(1.5)

        plt.tight_layout()  # 서브플롯 간 간격 조절
        plt.savefig(reports_dir + f'/us_b0100_{ticker}.png')






    '''
    2. Fundermental Analysis
    이것도 그래프로. (bar plot)
    '''


    '''
    3. Report
    '''



# Copyright 2023-2025 Jeongmin Kang, jarvisNim @ GitHub
# See LICENSE for details.

__author__ = "jarvisNim in GitHub"
__version__ = "0.0.5"


from .base import (
    get_stock_history_by_fmp,
    get_stock_history_by_yfinance,
    trend_detector,
    trend_detector_for_series,
    get_working_day_before,
)

from .tech import (
    get_tech_types,
    get_tech_hist,
    get_tech_fin,    
    make_tech_plot,
    sma,
    ema,
    macd,
    adx,
    psar,
    ichmoku,
    rsi,
    stoch,
    roc,
    cci,
    willr,
    ao,
    stochrsi,
    ppo,
    obv,
    pvt,
    pvi,
    cmf,
    vwap,
    adosc,
    mfi,
    kvo,
    nvi,
    atr,
    bbands,
    donchian,
    kc,
    rvi,
)

from .strategy import (
    sma_strategy,
    timing_strategy,
    get_vb_signals,
    show_vb_stategy_result,
    volatility_bollinger_strategy,
    get_reversal_signals,
    show_reversal_stategy_result,
    reversal_strategy,
    trend_following_strategy,
    control_chart_strategy,
    vb_genericAlgo_strategy,
    vb_genericAlgo2_strategy,
    gaSellHoldBuy_strategy,
)




'''
# PyPi
python setup.py sdist bdist_wheel
python -m twine upload dist/*
    (To use this API token:
    Set your username to __token__
    Set your password to the token value, including the pypi- prefix
    pypi-AgEIcHlwaS5vcmcCJGU3YmMxM2NiLTVhODctNGQ3My05NDZhLWQ0NjAwODA2MWQ2ZgACEVsxLFsicHltaW5lcnZhIl1dAAIsWzIsWyI4MWRjZDdlNC00NWEzLTQ1NGItOTU2ZC01YmNiN2UxZGNlOTciXV0AAAYgVcGZNNHlCozml9mNqeaab7amG5i_rz8Uc3uasXuyAOM
    )
python -m pip install pyminerva
# https://pypi.org/project/pyminerva/0.0.X/
pip install pyminerva --upgrade

# test PyPi
python setup.py sdist bdist_wheel
python -m twine upload --repository testpypi dist/*
    (To use this API token:
    Set your username to __token__
    Set your password to the token value, including the pypi- prefix)
python -m pip install --index-url https://test.pypi.org/simple/ --no-deps minerv
https://test.pypi.org/project/minerv/

'''

from setuptools import setup, find_packages
from os.path import abspath, dirname, join

# Fetches the content from README.md
# This will be used for the "long_description" field.
README_MD = open(join(dirname(abspath(__file__)), "README.md")).read()


with open('requirements.txt') as f:
    required = f.read().splitlines()

setup(
    name='pyminerva',
    version='0.0.230',    # version.directory.file
    description='To get an insight from Financial Data Anlaysis',
    url='',
    author='Jeongmin Kang',
    author_email='jarvisNim@gmail.com',
    license='MIT',
    # packages=['minerv'],
    # install_requires=required(filename='requirements.txt'),
    include_package_data=True,
    # url="https://github.com/driscollis/arithmetic",
    packages=find_packages(exclude="tests"),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    keywords=', '.join([
        'minerva', 'minerva-api', 'historical-data',
        'financial-data', 'stocks', 'funds', 'etfs',
        'indices', 'currency crosses', 'bonds', 'commodities',
        'crypto currencies'
    ]),
    project_urls={
        'Bug Reports': 'https://github.com/jarvisNim/minerva/issues',
        'Source': 'https://github.com/jarvisNim/minerva',
        'Documentation': 'https://miraelabs.com/'
    },
    python_requires='>=3.6',
)


import unittest

'''
# Prgram 명: make Macro Economic Tables in Economics Database
# 목적: 매일 올라오는 주요 국가별 경제지표들을 테이블에 추가하는 배치작업용
# Author: Jeonmin kang
# Mail: jarvisNim@gmail.com
# History
- 20231107  Create
- 20240209  OECD update 추가
'''

import sys, os
utils_dir = os.getcwd() + '/batch/utils'
sys.path.append(utils_dir)

from settings import *

import re
import requests
from bs4 import BeautifulSoup as bs
import yfinance as yf
import os
import webbrowser
import shutil
import platform


'''
공통 영역
'''
# logging
logger.warning(sys.argv[0] + ' :: ' + str(datetime.today()))
logger2.info('')
logger2.info(sys.argv[0] + ' :: ' + str(datetime.today()))
logger2.info('')

# 검색기간 설정
_to_date = pd.to_datetime(to_date, format="%d/%m/%Y")
term_days = relativedelta(days=5)  # 초기 작업시는 12주 로 하면 사이트 부하오류 발생 안해서 최적! 평소에는 1주기간 데이터만으로도 가능
from_date = (_to_date - term_days).date()
to_date = _to_date.date()

download_directory = "./batch/reports/data"

'''
데이터 베이스와 거시 경제 테이블 생성
'''
# Connect DataBase
database = database_dir+'/'+'Economics.db'
conn, engine = create_connection(database)


# 테이블 정의

str_alpha = '(\
    Country TEXT NOT NULL,\
    Market TEXT NOT NULL,\
    Busi TEXT NOT NULL,\
    Researcher TEXT NOT NULL,\
    Date TEXT NOT NULL,\
    Country_Growth NUMERIC,\
    Market_Growth NUMERIC,\
    Busi_Growth NUMERIC,\
    Trend NUMERIC,\
    Trend_3mo NUMERIC,\
    Trend_6mo NUMERIC,\
    Trend_12mo NUMERIC,\
    Trend_18mo NUMERIC,\
    Trend_24mo NUMERIC,\
    PRIMARY KEY (Country, Market, Busi, Researcher, Date))'


str_calendars = '(\
        date	TEXT NOT NULL,\
        country	TEXT NOT NULL,\
        event	TEXT NOT NULL,\
        currency	TEXT,\
        previous	NUMERIC,\
        estimate	NUMERIC,\
        actual	NUMERIC,\
        change	NUMERIC,\
        impact	TEXT,\
        changePercentage	NUMERIC,\
        PRIMARY KEY(date,country,event))'

str_markets = '(\
    Country TEXT NOT NULL,\
    Market TEXT NOT NULL,\
    Symbol TEXT NOT NULL,\
    Last_value NUMERIC NOT NULL, \
    Momentum NUMERIC, \
    Trend NUMERIC, \
    Oscillator NUMERIC, \
    RSI NUMERIC, \
    DOD NUMERIC, \
    WOW NUMERIC, \
    MOM NUMERIC, \
    YOY NUMERIC, \
    Date TEXT,\
    PRIMARY KEY (Country, Market, Symbol, Last_value))'

str_indicators = '(\
    Country TEXT NOT NULL,\
    Indicator TEXT NOT NULL,\
    Date TEXT NOT NULL,\
    Symbol TEXT,\
    Actual NUMERIC,\
    Previous NUMERIC,\
    MOM NUMERIC,\
    YOY NUMERIC,\
    Trend TEXT,\
    Slope TEXT,\
    Percentile INTEGER,\
    PRIMARY KEY (Country, Indicator, Date))'

str_stock_indices = '(\
    symbol TEXT NOT NULL,\
    name TEXT NOT NULL,\
    price NUMERIC,\
    changesPercentage NUMERIC,\
    change NUMERIC,\
    Previous NUMERIC,\
    dayLow NUMERIC,\
    dayHigh NUMERIC,\
    yearHigh NUMERIC,\
    yearLow NUMERIC,\
    marketCap NUMERIC,\
    priceAvg50 NUMERIC,\
    priceAvg200 NUMERIC,\
    exchange TEXT,\
    volume INTEGER,\
    avgVolume INTEGER,\
    open NUMERIC,\
    previousClose NUMERIC,\
    eps  NUMERIC,\
    pe   NUMERIC,\
    earningsAnnouncement    NUMERIC,\
    sharesOutstanding   NUMERIC,\
    timestamp INTEGER NOT NULL,\
    PRIMARY KEY (symbol, name, timestamp))'


'''
financemodeling.com 추출한 경제 캘린더 <=== 검증완료: 수기 작성과 어플리케이션 작성 모두 멀티 Primary key 구성 가능
'''
def create_Calendars(conn, str_calendar):
    with conn:
        cur = conn.cursor()
        cur.execute(f'CREATE TABLE if not exists Calendars {str_calendars}')
    return conn

'''
MacroVar.com 에서 추출한 Financial Markets <== 현재는 사이트의 정보 업데이트가 미흡하여 미활용, 주기적으로 확인필요
'''
def create_Markets(conn, str_markets):
    with conn:
        cur = conn.cursor()
        cur.execute(f'CREATE TABLE if not exists Markets {str_markets}')
    return conn

'''
MacroVar.com 에서 추출한 Macroeconomic Indicators
'''
def create_Indicators(conn, str_indicators):
    with conn:
        cur = conn.cursor()
        cur.execute(f'CREATE TABLE if not exists Indicators {str_indicators}')

    return conn

'''
financialmodelingprep.com 에서 추출한 Stock Market Indices
'''
def create_Stock_Indices(conn, str_indicators):
    with conn:
        cur = conn.cursor()
        cur.execute(f'CREATE TABLE if not exists Stock_Indices {str_stock_indices}')

    return conn

'''
각종 Research 에서 추출한 Trend, Country/Market/Business Growth 를 가지고 현 사이클 단계 확인과 6,12개월 전망
- Researcher: OECD, IMF, Tech, Senti >> Total
- Stauts: Buttom -> Bull-1 -> Bull-2 -> Top -> Bear-1 -> Bear-2
'''
def create_Alpha(conn, str_alpha):
    with conn:
        cur = conn.cursor()
        cur.execute(f'CREATE TABLE if not exists Alpha {str_alpha}')
    return conn


# 테이블 데이터 read
def read_table(table_name):
    M_table = table_name
    M_query = f"SELECT * from {M_table}"
    try:
        buf = pd.read_sql_query(M_query, conn)
        # logger2.info(buf)
        return buf
    except Exception as e:
        logger.error(' >>> Exception: {}'.format(e))

# 테이블 데이터 insert
def write_table(table_name, data):
    insert_count = 0
    delete_count = 0

    if table_name == 'Calendars':
        data.dropna(subset=['date', 'country', 'event'], inplace=True)     
    elif table_name == 'Markets':
        data.dropna(subset=['Country', 'Market', 'Symbol', 'Last_value'], inplace=True) 
    elif table_name == 'Indicators':
        data.dropna(subset=['Country', 'Indicator', 'Date'], inplace=True)
    elif table_name == 'Stock_Indices':
        data.dropna(subset=['symbol', 'name', 'timestamp'], inplace=True)
    elif table_name == 'Alpha':
        data.dropna(subset=['Country', 'Market', 'Busi', 'Researcher', 'Date'], inplace=True)
    else:
        logger.error(' >>> Exception: Table Name Not found.')

    data = data.reset_index(drop=True) 

    # print(data[70:120])
    with conn:
        cur=conn.cursor()
        for i, row in data.iterrows():

            buf = data.iloc[i:i+1, :]
            if table_name == 'Calendars':
                _key = f"date = '{buf['date'][i]}' and country = '{buf['country'][i]}' and event = '{buf['event'][i]}'"     
            elif table_name == 'Markets':
                _key = f"Country = '{buf['Country'][i]}' and Symbol = '{buf['Symbol'][i]}' and Last_value = '{buf['Last_value'][i]}'"
            elif table_name == 'Indicators':
                _key = f"Country = '{buf['Country'][i]}' and Indicator = '{buf['Indicator'][i]}' and Date = '{buf['Date'][i]}'"
            elif table_name == 'Stock_Indices':
                _key = f"symbol = '{buf['symbol'][i]}' and name = '{buf['name'][i]}' and timestamp = '{buf['timestamp'][i]}'"
            elif table_name == 'Alpha':
                _key = f"Country = '{buf['Country'][i]}' and Market = '{buf['Market'][i]}' and Busi = '{buf['Busi'][i]}' and Researcher = '{buf['Researcher'][i]}' and Date = '{buf['Date'][i]}'"

            else:
                logger.error(' >>> Exception: Table Name Not found 2.')            

            # print(f'delete from {table_name} where {_key}')  # DEBUG
            cur.execute(f"delete from {table_name} where {_key}")
            cur.execute('commit')
            delete_count += 1

            # print('insert...', buf)     # DEBUG
            _cnt = buf.to_sql(table_name, con=engine, if_exists='append', chunksize=1000, index=False)

            insert_count += _cnt

    logger2.info('')
    logger2.info(f'{table_name} delete Count: ' + str({delete_count}))
    logger2.info(f'{table_name} insert Count: ' + str({insert_count}))
    logger2.info('')


# 덤프 테이블 데이터 replace
def write_dump_table(table_name, data):
    insert_count = 0

    _cnt = data.to_sql(table_name, con=engine, if_exists='replace', chunksize=1000, index=False, method='multi')
    
    insert_count += _cnt
    logger2.info('')    
    logger2.info(f'Dump {table_name} replace Count: ' + str({insert_count}))
    logger2.info('')    


# macrovar.com 에서 markets 표 읽어오기
def get_markets(country, url, table_name):
    page = requests.get(url, allow_redirects=True)
    soup = bs(page.text, "html.parser")
    table = soup.find_all(class_="container--tabs mb-2")

    # 1. Markets
    markets = pd.read_html(str(table))[0]

    buf = pd.DataFrame()
    buf['Country'] = [country] * len(markets)
    buf['Market'] = markets.Market
    buf['Symbol'] = markets.Symbol
    buf['Last_value'] = markets.Last
    buf['Momentum'] = markets.Mom
    buf['Trend'] = markets.Trend
    buf['Oscillator'] = markets.Exh
    buf['RSI'] = markets.RSI
    buf['DOD'] = markets['1D%']
    buf['WOW'] = markets['1W%']    
    buf['MOM'] = markets['1M%']
    buf['YOY'] = markets['1Y%']    
    buf['Date'] = pd.to_datetime(to_date2).date()

    return buf

# macrovar.com 에서 indicator 표 읽어오기
def get_indicators(country, url, table_name):
    page = requests.get(url, allow_redirects=True)
    soup = bs(page.text, "html.parser")
    table = soup.find_all(class_="container--tabs mb-2")

    # Macroeconomic Indicators
    indicators = pd.read_html(str(table))[1]
    indicators['Country'] = [country] *  len(indicators)
    indicators['Date'] = pd.to_datetime(indicators['Update']).dt.date

    buf = pd.DataFrame()
    buf['Country'] = indicators.Country
    buf['Indicator'] = indicators.Indicator
    buf['Date'] = indicators.Date
    buf['Symbol'] = indicators.Symbol
    buf['Actual'] = indicators.Actual
    buf['Previous'] = indicators.Previous
    buf['MOM'] = indicators['M/M%']
    buf['YOY'] = indicators['Y/Y%']
    buf['Trend'] = indicators.Trend
    buf['Slope'] = indicators.Slope
    buf['Percentile'] = indicators['Percentile']

    return buf



'''
1. Calendars 테이블 데이터 구성
'''
def make_calendars(from_date, to_date):
    table_name = 'Calendars'
    cals = pd.DataFrame()
    for i in range(1):  # 최초 구성시는 20? 이후 매일 3회 배치작업으로 구성하고 있으니 1바퀴만 돌면 괜찮을듯.
        buf = get_calendar(from_date=from_date, to_date=to_date)
        buf = buf[buf['event'] != "New Year's Day"] # New Year's Day 제거, 's 로 syntax error 유발
        buf = buf[buf['event'] != "New Year's Eve"] # New Year's Eve 제거, 's 로 syntax error 유발 
        # print(buf)
        for i in range(len(NATIONS)):
            buf2 = buf[buf['country'] == NATIONS[i]]
            cals = pd.concat([cals, buf2], axis=0)
        to_date = pd.to_datetime(from_date)
        from_date = (to_date - term_days).date()
        to_date = to_date.date()

    cals = cals.reset_index(drop=True)

    logger2.info('')
    logger2.info(f' 최근 5일동안의 Calendars 중 estimate 까지 발표된 지표 '.center(60, '*'))
    buffer = cals[pd.to_datetime(cals['date']) >=  pd.to_datetime(to_date)]
    buffer = buffer.dropna(subset=['estimate'])  # 컨센서스가 지난 대비 긍정 또는 부정으로 판단하는지 볼수있는 estimate 도 포함.
    logger2.info(buffer.sort_values(by='date', ascending=False).to_string())    
    write_table(table_name, cals)


'''
2. Markets 테이블 데이터 구성
- macrovar.com 에서 markets 표 읽어오기
'''
def make_markets(**kwargs):
    table_name = 'Markets'
    df = pd.DataFrame()
    for key, value in kwargs.items():
        buf = get_markets(key, value, table_name)
        logger2.info('')
        logger2.info(f' macrovar.com 의 markets 표: {buf.Country[0]} '.center(60, '*'))
        logger2.info(buf)        
        df = pd.concat([df, buf])

    df = df.reset_index(drop=True)     
    write_table(table_name, df)

'''
3. Indicators 테이블 데이터 구성
- macrovar.com 에서 indicator 표 읽어오기
'''
def make_indicators(**kwargs):
    table_name = 'Indicators'
    df = pd.DataFrame()
    for key, value in kwargs.items():
        buf = get_indicators(key, value, table_name)
        logger2.info('')
        logger2.info(f'macrovar.com 의 indicator 표: {buf.Country[0]} '.center(60, '*'))
        logger2.info(buf)          
        df = pd.concat([df, buf])

    df = df.reset_index(drop=True)
    write_table(table_name, df)


'''
4. Global Stock Market Indices 테이블 데이터 구성
'''
def make_stock_indices(**kwargs):
    table_name = 'Stock_Indices'
    df = get_stock_indices()
    df = df.reset_index(drop=True)
    write_table(table_name, df)


'''
5. IMF Data 구성
https://www.imf.org/en/Publications/SPROLLS/world-economic-outlook-databases#sort=%40imfdate%20descending 사이트에서 
최신 업데이트 정보목록을 선택하여 들어간후, Entire Dataset 버튼 > By Country 버튼 누르면 다운로드 완료.
Download 사이트에서 해당 파일의 이름을 IMF_outlook.xls 로 변경후 batch/reports/data/으로 이동한후 DB 업로드 프로그램 수행
- 20240103 https://www.imf.org/-/media/Files/Publications/WEO/WEO-Database/2023/WEOOct2023all.ashx 
- 20240131 현재 https://www.imf.org/-/media/Files/Publications/WEO/2024/Update/January/English/Data/WEOJAN2024Update.ashx
'''
def make_imf_outlook():

    table_name = 'IMF'

    # url = 'https://www.imf.org/-/media/Files/Publications/WEO/WEO-Database/2023/WEOOct2023all.ashx'
    url = 'https://www.imf.org/-/media/Files/Publications/WEO/WEO-Database/2023/WEOOct2023all.ashx'    

    response = requests.get(url, timeout=10, verify=False)
    
    if response.status_code == 200:
        with open(download_directory+'/IMF.csv', 'wb') as file:
            file.write(response.content)
        sleep(10)            
    else:
        logger.error(f" >>> WorldBank.xlsx 다운로드 실패. 응답 코드: {response.status_code}")

    df = pd.read_csv('./batch/reports/data/IMF.csv', sep='\t', skip_blank_lines=True, skipfooter=3, encoding_errors='replace')
    df = df.reset_index(drop=True)
    write_dump_table(table_name, df)



'''
6. OECD Data 구성
- ref: https://medium.com/@akshaybagal/oecd-stats-website-your-go-to-for-comprehensive-statistics-datasets-on-oecd-countries-2ae04e4aa044
- OECD 아래 사이트 들어가서 > Export > Developer API > 'Generate API Queries' 버튼 눌러서 나온 url 으로 구성, # 2025년 까지를 2030년으로 길게 늘려둠. ^^
  https://stats.oecd.org/Index.aspx?DataSetCode=EO
- id_to_name_mappings:
{'Country': {'AUS': 'Australia',
  'AUT': 'Austria',
  'BEL': 'Belgium',
  'CAN': 'Canada',
  'CZE': 'Czechia',
  'DNK': 'Denmark',
  'FIN': 'Finland',
  'FRA': 'France',
  'DEU': 'Germany',
  'GRC': 'Greece',
  'HUN': 'Hungary',
  'ISL': 'Iceland',
  'IRL': 'Ireland',
  'ITA': 'Italy',
  'JPN': 'Japan',
  'KOR': 'Korea',
  'LUX': 'Luxembourg',
  'MEX': 'Mexico',
  'NLD': 'Netherlands',
  'NZL': 'New Zealand',
  'NOR': 'Norway',
  'POL': 'Poland',
  'PRT': 'Portugal',
  'SVK': 'Slovak Republic',
  'ESP': 'Spain',
  'SWE': 'Sweden',
  'CHE': 'Switzerland',
  'TUR': 'Türkiye',
  'GBR': 'United Kingdom',
  'USA': 'United States',
  'OTO': 'OECD - Total',
  'BRA': 'Brazil',
  'CHL': 'Chile',
  'CHN': "China (People's Republic of)",
  'EST': 'Estonia',
  'IND': 'India',
  'IDN': 'Indonesia',
  'ISR': 'Israel',
  'RUS': 'Russia',
  'SVN': 'Slovenia',
  'ZAF': 'South Africa',
  'WLD': 'World',
  'DAE': 'Dynamic Asian Economies',
  'OOP': 'Other oil producers',
  'COL': 'Colombia',
  'LVA': 'Latvia',
  'NMEC': 'Non-OECD Economies',
  'LTU': 'Lithuania',
  'CRI': 'Costa Rica',
  'ARG': 'Argentina',
  'ROU': 'Romania',
  'BGR': 'Bulgaria',
  'HRV': 'Croatia',
  'PER': 'Peru'},
 'Variable': {'CBGDPR': 'Current account balance as a percentage of GDP',
  'EXCHUD': 'Exchange rate, national currency per USD',
  'MGSVD': 'Imports of goods and services, volume in USD (national accounts basis)',
  'TGSVD': 'Goods and services trade, volume in USD',
  'XGSVD': 'Exports of goods and services, volume in USD (national accounts basis)',
  'NLGQ': 'General government net lending as a percentage of GDP',
  'GGFLQ': 'General government gross financial liabilities as a percentage of GDP',
  'GDP': 'Gross domestic product, nominal value, market prices',
  'GDPV': 'Gross domestic product, volume, market prices',
  'ITV': 'Gross fixed capital formation, total, volume',
  'CGV': 'Government final consumption expenditure, volume',
  'CPV': 'Private final consumption expenditure, volume',
  'XGSV_ANNPCT': 'Exports of goods and services, volume, growth (national accounts basis)',
  'MGSV_ANNPCT': 'Imports of goods and services, volume, growth (national accounts basis)',
  'PGDP_ANNPCT': 'Gross domestic product, market prices, deflator, growth',
  'PCORE_YTYPCT': 'Core inflation',
  'CPV_ANNPCT': 'Private final consumption expenditure, volume, growth',
  'CPI_YTYPCT': 'Headline inflation',
  'CGV_ANNPCT': 'Government final consumption expenditure, volume, growth',
  'ITV_ANNPCT': 'Gross fixed capital formation, total, volume, growth',
  'GDPV_ANNPCT': 'Gross domestic product, volume, growth',
  'CPI': 'Consumer price index',
  'PXGS': 'Exports of goods and services, deflator (national accounts basis)',
  'PMGS': 'Imports of goods and services, deflator (national accounts basis)',
  'PGDP': 'Gross domestic product, market prices, deflator',
  'UNR': 'Unemployment rate',
  'ET': 'Total employment (labour force survey basis)',
  'LF': 'Labour force',
  'IRL': 'Long-term interest rate on government bonds',
  'IRS': 'Short-term interest rate',
  'GGFLMQ': 'Gross public debt, Maastricht criterion as a percentage of GDP',
  'PCOREH_YTYPCT': 'Harmonised core inflation',
  'CPIH_YTYPCT': 'Harmonised headline inflation',
  'CPIH': 'Consumer price index, harmonised',
  'WPBRENT': 'Crude oil price, FOB, USD per barrel, spot Brent',
  'PCOREH': 'Core inflation index, harmonised',
  'MGSV': 'Imports of goods and services, volume (national accounts basis)',
  'PCORE': 'Core inflation index',
  'XGSV': 'Exports of goods and services, volume (national accounts basis)',
  'CQ_FBGSV': 'Net exports, contributions to changes in real GDP',
  'GDPVD': 'Gross domestic product, volume in USD, at constant purchasing power parities',
  'GDP_ANNPCT': 'Gross domestic product, nominal value, growth',
  'EXCH': 'Exchange rate, USD per national currency',
  'YDH': 'Net disposable income of households and non-profit institutions serving households',
  'NLGXQ': 'General government primary balance as a percentage of GDP',
  'PPP': 'Purchasing power parity, national currency per USD',
  'CQ_ISKV': 'Change in inventories, contributions to changes in real GDP',
  'ITISKV': 'Gross capital formation, total, volume',
  'GFAR': 'General government gross financial assets as a percentage of GDP',
  'YDH_G': 'Gross disposable income of household and non-profit institutions serving households',
  'GDPML': 'Mainland gross domestic product , nominal value, market prices',
  'GDPMLV': 'Mainland gross domestic product, volume, market prices',
  'GDPV_USD': 'Gross domestic product, volume in USD, constant exchange rates'},
 'Frequency': {'A': 'Annual'},
 'Time': {'2016': '2016',
  '2017': '2017',
  '2018': '2018',
  '2019': '2019',
  '2020': '2020',
  '2021': '2021',
  '2022': '2022',
  '2023': '2023',
  '2024': '2024',
  '2025': '2025'}}

6.1 Interim report update function: update_oecd_interim()
'''
def make_oecd_outlook():

    table_name = 'OECD'

    url = "https://stats.oecd.org/SDMX-JSON/data/EO/AUS+AUT+BEL+CAN+CHL+COL+CRI+CZE+DNK+EST+FIN+FRA+DEU+GRC+HUN+ISL+IRL+ISR+ITA+JPN+KOR+LVA+LTU+LUX+MEX+NLD+NZL+NOR+POL+PRT+SVK+SVN+ESP+SWE+CHE+TUR+GBR+USA+OTO+WLD+NMEC+ARG+BRA+BGR+CHN+HRV+IND+IDN+PER+ROU+RUS+ZAF+DAE+OOP.EXT+CBGDPR+EXCHUD+EXCH+XGSVD+TGSVD+MGSVD+GOV+GFAR+GGFLQ+NLGQ+NLGXQ+GGFLMQ+EXP+CQ_ISKV+XGSV+CGV+ITISKV+GDP+GDPVD+GDPV_USD+GDPV+ITV+MGSV+GDPML+GDPMLV+CQ_FBGSV+CPV+SEL+PCORE_YTYPCT+XGSV_ANNPCT+CGV_ANNPCT+PGDP_ANNPCT+GDP_ANNPCT+GDPV_ANNPCT+ITV_ANNPCT+PCOREH_YTYPCT+CPIH_YTYPCT+CPI_YTYPCT+MGSV_ANNPCT+CPV_ANNPCT+HOU+YDH_G+YDH+PRI+CPI+CPIH+PCORE+PCOREH+PXGS+PGDP+PMGS+PPP+LAB+LF+ET+UNR+MON+IRL+IRS+OIL+WPBRENT.A/all?startTime=2016&endTime=2030&dimensionAtObservation=allDimensions"
    response = requests.request("GET", url)

    data = response.json()
    data_values = data['dataSets'][0]['observations']
    dimensions = data['structure']['dimensions']['observation']

    id_to_name_mappings = {
        dim['name']: {item['id']: item['name'] for item in dim['values']}
        for dim in dimensions
    }

    dimension_values = [dim['values'] for dim in dimensions]

    def get_id_from_index(dim_index, index):
        return dimension_values[dim_index][index]['id']

    def map_id_to_name(dim_name, id):
        return id_to_name_mappings[dim_name].get(id, id)

    rows = []
    for key, value in data_values.items():
        indices = key.split(':')  # Split keys into separate dimension indices
        country = map_id_to_name('Country', get_id_from_index(0, int(indices[0])))
        variable = map_id_to_name('Variable', get_id_from_index(1, int(indices[1])))
        frequency = map_id_to_name('Frequency', get_id_from_index(2, int(indices[2])))  # Corrected typo here
        time = map_id_to_name('Time', get_id_from_index(3, int(indices[3])))
        data_value = value[0]  # Extract the data value
        rows.append([country, variable, frequency, time, data_value])  # Append the row to the list of rows

    df = pd.DataFrame(rows, columns=['Country', 'Variable', 'Frequency', 'Time', 'Value'])
    write_dump_table(table_name, df)

    return df


'''
대상 indicator 는 inflation, gdp 
- 20240207: OECD Economic Outlook, Interim Report February 2024: inflation, gdp
- SELECT * FROM OECD WHERE Variable like '%Gross domestic product, volume, growth%' AND (Time = '2024' or Time = '2025')
- SELECT * FROM OECD WHERE Variable like '%Headline inflation%' AND (Time = '2024' or Time = '2025')
'''
def update_oecd_interim(conn):
    variable_gdp = "%Gross domestic product, volume, growth%"  # Source: OECD Economic Outlook, Interim Report February 2024.
    variable_inflation = "%Headline inflation%"  # Headline Inflation, Source: OECD Economic Outlook, Interim Report February 2024.
    gdp_update =     {
        'Brazil%':[{'2024':1.81771723436261}, {'2025':2.02464923513675}],
        'China%': [{'2024':4.68607906868843}, {'2025':4.24280944230372}],
        'Euro%': [{'2024':0.635063372653745}, {'2025':1.33172323471308}],        
        'Germany%': [{'2024':0.279680495326152}, {'2025':1.10892974166749}],
        'India%': [{'2024':6.15974810807787}, {'2025':6.51778646588327}],
        'Japan%': [{'2024':0.997774345949994}, {'2025':0.998694183827788}],
        'Korea%': [{'2024':2.16269397354112}, {'2025':2.11686859085889}],
        'United States%': [{'2024':2.1488822258862}, {'2025':1.71411734194476}],
    }

    inflation_update =     {
        'Brazil%':[{'2024':3.28414019019374}, {'2025':2.99603206276178}],
        'China%': [{'2024':1.04729033978783}, {'2025':1.45639190001296}],
        'Euro%': [{'2024':0.635063372653745}, {'2025':1.33172323471308}],        
        'Germany%': [{'2024':2.58814674906734}, {'2025':2.01387974466703}],
        'India%': [{'2024':4.91947232367986}, {'2025':4.25901301129225}],
        'Japan%': [{'2024':2.59695374744398}, {'2025':2.04680754163396}],
        'Korea%': [{'2024':2.66138758411523}, {'2025':2.03949651385258}],
        'United States%': [{'2024':2.17034237341851}, {'2025':1.9958871662692}],
    }    

    with conn:
        cur=conn.cursor()
        update_count = 0
        for country in gdp_update.keys():
            # print(x)
            # print(oecd_update[x])
            for i, gdp in enumerate(gdp_update[country]):
                key = list(gdp.keys())
                year = key[0]
                value = gdp[key[0]]
                # print(year)
                # print(value)
                if i == 0:
                    second_year = str(int(year)+1)
                    # print(f"SELECT * FROM OECD WHERE Country like '{country}' AND Variable like '{variable_gdp}' AND (Time = '{year}' or Time = '{second_year}') ")
                    cur.execute(f"SELECT * FROM OECD WHERE Country like '{country}' AND Variable like '{variable_gdp}' AND (Time = '{year}' or Time = '{second_year}') ")

                # print(f"update OECD set Value = {value} where Country like '{country}' AND Variable like '{variable_gdp}' and Time = {year} ")
                cur.execute(f"update OECD set Value = {value} where Country like '{country}' AND Variable like '{variable_gdp}' and Time = {year} ")
                update_count += 1
        logger2.info(f"GDP Growth update count: {update_count}") 

        # Inflation update
        update_count = 0
        for country in inflation_update.keys():
            # print(x)
            # print(oecd_update[x])
            for i, inflation in enumerate(inflation_update[country]):
                key = list(inflation.keys())
                year = key[0]
                value = inflation[key[0]]
                # print(year)
                # print(value)
                if i == 0:
                    second_year = str(int(year)+1)
                    # print(f"SELECT * FROM OECD WHERE Country like '{country}' AND Variable like '{variable_inflation}' AND (Time = '{year}' or Time = '{second_year}') ")
                    cur.execute(f"SELECT * FROM OECD WHERE Country like '{country}' AND Variable like '{variable_inflation}' AND (Time = '{year}' or Time = '{second_year}') ")

                # print(f"update OECD set Value = {value} where Country like '{country}' AND Variable like '{variable_inflation}' and Time = {year} ")
                cur.execute(f"update OECD set Value = {value} where Country like '{country}' AND Variable like '{variable_inflation}' and Time = {year} ")
                update_count += 1
        logger2.info(f"Inflation update count: {update_count}")            
        
        cur.execute('commit')
        


'''
7. WorldBank Data 생성
https://www.worldbank.org/en/news/press-release/2024/01/09/global-economic-prospects-january-2024-press-release?intcid=ecr_hp_headerA_2024-01-09-GEPPressRelease
사이트에서  (향후 하반기 업데이트시는 다른 사이트일듯.), 'Download growth data' 버튼 누르면 다운로드 완료.
Download 사이트에서 해당 파일의 이름을 WorldBank.xls 로 변경후 batch/reports/data/으로 이동한후 DB 업로드 프로그램 수행
- 20240110 현재 https://bit.ly/GEP-Jan-2024-GDP-growth-data 파일이 최신임
'''
def make_worldbank_outlook():

    table_name = 'WorldBank'

    url = 'https://bit.ly/GEP-Jan-2024-GDP-growth-data'

    response = requests.get(url, timeout=10, verify=False)
    
    if response.status_code == 200:
        with open(download_directory+'/WorldBank.xlsx', 'wb') as file:
            file.write(response.content)
        sleep(10)            
    else:
        logger.error(f" >>> WorldBank.xlsx 다운로드 실패. 응답 코드: {response.status_code}")

    df = pd.read_excel('./batch/reports/data/WorldBank.xlsx', skiprows=range(0, 3),)
    df = df.reset_index(drop=True)
    df.columns = ['Category_1', 'Category_2', 'Category_3', 'Category_4', 'Category_5', '_2021', '_2022', '_2023e',\
                  '_2024f', '_2025f', 'filler' ,'_2023e_d', '_2024f_d', '_2025f_d']
    write_dump_table(table_name, df)



'''
8. Alpha 테이블 생성
glbal_.py 프로그램 수행하면서 발생되는 국가/시장/사업별로 저장하는 루틴
각종 Researcher 에서 추출한 Trend, Country/Market/Business Growth 를 가지고 현 사이클 단계 확인과 6,12, 18, 24개월 전망 지표 보여줌.
- Researcher: OECD, IMF, WorldBank, Tech.., Senti.. >> Total
'''
def make_alpha(data):
    table_name = 'Alpha'
    # print(data)
    df = data
    df = df.dropna()
    df = df.reset_index(drop=True)
    write_table(table_name, df)


'''
999. 주기적 insert/delete 작업으로 키값 저장위치가 분산되어 그래프화시 노이즈 발생하여 이를 제거하기 위하여 재구성함.
- cron 작업 첫번째로 순서 변경
'''
def reorg_tables(conn):

    cur = conn.cursor()

    # 테이블 목록 조회
    cur.execute("SELECT name FROM sqlite_master WHERE type='table';")
    tables = cur.fetchall()
    for table in tables:
        logger2.info(f'Reorg table names: {table[0]}')
    logger2.info('')


    # 1. Markets 테이블 재구성
    try:
        cur.execute('DROP TABLE Markets_backup;')
    except Exception as e:
        logger.error(' >>> Exception: {}'.format(e))
        pass
    cur.execute(f'CREATE TABLE Markets_backup {str_markets};')
    cur.execute('INSERT INTO Markets_backup SELECT * FROM Markets ORDER BY Date;')    
    cur.execute('DROP TABLE Markets;')

    cur.execute(f'CREATE TABLE Markets {str_markets};')
    cur.execute(f'INSERT INTO Markets SELECT * FROM Markets_backup;')    
    cur.execute('SELECT count(*) FROM Markets;')
    result = cur.fetchone()      
    logger2.info(f'Markets Reorg Count: ' + str(result[0]))    # result에는 (행의 수,) 형태의 튜플이 들어 있습니다.
    conn.commit()

    # 2. Indicators 테이블 재구성    
    try:
        cur.execute('DROP TABLE Indicators_backup;')
    except Exception as e:
        logger.error(' >>> Exception: {}'.format(e))
        pass
    cur.execute(f'CREATE TABLE Indicators_backup {str_indicators};')
    cur.execute('INSERT INTO Indicators_backup SELECT * FROM Indicators ORDER BY Country, Indicator, Date;')    
    cur.execute('DROP TABLE Indicators;')

    cur.execute(f'CREATE TABLE Indicators {str_indicators};')
    cur.execute(f'INSERT INTO Indicators SELECT * FROM Indicators_backup;')    
    cur.execute('SELECT count(*) FROM Indicators;')
    result = cur.fetchone()      
    logger2.info(f'Indicators Reorg Count: ' + str(result[0]))    # result에는 (행의 수,) 형태의 튜플이 들어 있습니다.
    conn.commit()

    # 3. Calendars 테이블 재구성
    try:
        cur.execute('DROP TABLE Calendars_backup;')
    except Exception as e:
        logger.error(' >>> Exception: {}'.format(e))
        pass
    cur.execute(f'CREATE TABLE Calendars_backup {str_calendars};')
    cur.execute('INSERT INTO Calendars_backup SELECT * FROM Calendars ORDER BY country, date;')    
    cur.execute('DROP TABLE Calendars;')

    cur.execute(f'CREATE TABLE Calendars {str_calendars};')
    cur.execute(f'INSERT INTO Calendars SELECT * FROM Calendars_backup;')    
    cur.execute('SELECT count(*) FROM Calendars;')
    result = cur.fetchone()      
    logger2.info(f'Calendars Reorg Count: ' + str(result[0]))    # result에는 (행의 수,) 형태의 튜플이 들어 있습니다.
    conn.commit()

    # 4. Stock_Indices 테이블 재구성
    try:
        cur.execute('DROP TABLE Stock_Indices_backup;')
    except Exception as e:
        logger.error(' >>> Exception: {}'.format(e))
        pass
    cur.execute(f'CREATE TABLE Stock_Indices_backup {str_stock_indices};')
    cur.execute('INSERT INTO Stock_Indices_backup SELECT * FROM Stock_Indices ORDER BY symbol, name, timestamp;')    
    cur.execute('DROP TABLE Stock_Indices;')

    cur.execute(f'CREATE TABLE Stock_Indices {str_stock_indices};')
    cur.execute(f'INSERT INTO Stock_Indices SELECT * FROM Stock_Indices_backup;')    
    cur.execute('SELECT count(*) FROM Stock_Indices;')
    result = cur.fetchone()      
    logger2.info(f'Stock_Indices Reorg Count: ' + str(result[0]))    # result에는 (행의 수,) 형태의 튜플이 들어 있습니다.
    conn.commit()    

    return conn


'''
Main Fuction
'''

if __name__ == "__main__":

    '''
    # 테이블 생성 (최초 생성시만 Active 해서 사용 !!!)
    create_Alpha(conn, str_alpha)
    create_Calendars(conn, str_calendars)
    create_Markets(conn, str_markets)
    create_Indicators(conn, str_indicators)
    create_Stock_Indices(conn, str_stock_indices)
    '''

    '''
    # 테이블내 데이터 만들어 넣기
    '''
    try:
        make_calendars(from_date, to_date)
        make_markets(**urls)
        make_indicators(**urls)
        make_stock_indices()
        make_imf_outlook()
        make_oecd_outlook()
        update_oecd_interim(conn)
        make_worldbank_outlook()
    except Exception as e:
        logger.error(' >>> Exception: {}'.format(e))

    '''
    # make_alpha() 는 존재하지 않음.  _global.py 프로그램에서 호출해서 수행함.   
    '''



    '''
    # 테이블 저장공간 키구성순을 위한 재구성작업
    '''
    try:
        reorg_tables(conn)
    except Exception as e:
        logger.error(' >>> Exception: {}'.format(e))









